#include "Play.hpp"

using std::vector;

void Block::create_body(dWorldID world, dSpaceID space) 
{
	assert(body == NULL);
	assert(geom == NULL);

	body = dBodyCreate(world);
	{
		dMass mass;
		dMassSetBox(&mass, density, radius.x * 2, radius.y * 2, radius.z * 2);
		dBodySetMass(body, &mass);
	}

	// geom for body
	geom = dCreateBox(space, radius.x * 2, radius.y * 2, radius.z * 2);
	dGeomSetBody(geom, body);
	dBodySetPosition(body, position.x, position.y, position.z);
	dBodySetData(body, this);
	{
		dQuaternion quat;
		quat[1] = rotation[0];
		quat[2] = rotation[1];
		quat[3] = rotation[2];
		quat[0] = rotation[3];
		dBodySetQuaternion(body, quat);
	}
}

void Geometry::copy_body_position()
{
	const dReal *pos = dBodyGetPosition(body);
	position.x = pos[0];
	position.y = pos[1];
	position.z = pos[2];
	const dReal *quat = dBodyGetQuaternion(body);
	rotation[0] = quat[1];
	rotation[1] = quat[2];
	rotation[2] = quat[3];
	rotation[3] = quat[0];
}

void Geometry::force_body()
{
	dBodySetPosition(body, position.x, position.y, position.z);
	dQuaternion quat;
	quat[1] = rotation[0];
	quat[2] = rotation[1];
	quat[3] = rotation[2];
	quat[0] = rotation[3];
	dBodySetQuaternion(body, quat);
}

void Geometry::destroy_body() 
{
	assert(body);
	assert(geom);
	dGeomDestroy(geom);
	geom = NULL;
	dBodyDestroy(body);
	body = NULL;
}

void Sphere::create_body(dWorldID world, dSpaceID space) 
{
	assert(body == NULL);
	assert(geom == NULL);

	body = dBodyCreate(world);
	{
		dMass mass;
		dMassSetSphere(&mass, density, radius);
		dBodySetMass(body, &mass);
	}

	// geom for body
	geom = dCreateSphere(space, radius);
	dGeomSetBody(geom, body);
	dBodySetPosition(body, position.x, position.y, position.z);
	dBodySetData(body, this);
	{
		dQuaternion quat;
		quat[1] = rotation[0];
		quat[2] = rotation[1];
		quat[3] = rotation[2];
		quat[0] = rotation[3];
		dBodySetQuaternion(body, quat);
	}
}

void Play::startWorld() {
	assert(world == NULL);
	assert(space == NULL);
	assert(collisions == NULL);
	
	world = dWorldCreate();
	space = dHashSpaceCreate(NULL);
	// create plane with y = 0
	ground = dCreatePlane(space, 0.0f, 1.0f, 0.0f, 0.0);
	dGeomSetData(ground, (void*)GROUND_CONSTANT);
	collisions = dJointGroupCreate(0);

	dWorldSetGravity(world, 0.0f, -10.0f, 0.0f);

	sim_acc = 0.0f;
}

void Play::stopWorld() {
	assert(world != NULL);
	assert(space != NULL);
	assert(collisions != NULL);

	dGeomDestroy(ground);
	ground = NULL;
	dJointGroupDestroy(collisions);
	collisions = NULL;
	dSpaceDestroy(space);
	space = NULL;
	dWorldDestroy(world);
	world = NULL;
}

class Info {
public:
	dWorldID world;
	dJointGroupID contacts;
	vector< Block* > *wall;
	vector< Geometry* > *otherObjects;
};

void near_callback(void *data, dGeomID o1, dGeomID o2) 
{
	Info *info = (Info *) data;

	const unsigned int MAX_CONTACTS = 10;
	dContact contact[MAX_CONTACTS];
	for (unsigned int i = 0; i <MAX_CONTACTS; i++) {
		contact[i].surface.mode = dContactBounce | dContactSoftCFM;
		contact[i].surface.mu = dInfinity;
		contact[i].surface.mu2 = 0;
		contact[i].surface.bounce = 0.1;
		contact[i].surface.bounce_vel = 0.1;
		contact[i].surface.soft_cfm = 0.01;
	}
	dBodyID b1 = dGeomGetBody(o1);
	dBodyID b2 = dGeomGetBody(o2);
	bool collision_with_ground = (dGeomGetData(o1) == (void*)GROUND_CONSTANT || dGeomGetData(o2) == (void*)GROUND_CONSTANT);

	if (int numc = dCollide(o1, o2, MAX_CONTACTS, &contact[0].geom, sizeof(dContact))) 
	{
		if (b1 && b2)
		{
			Geometry *g1 = (Geometry *)dBodyGetData(b1);
			Geometry *g2 = (Geometry *)dBodyGetData(b2);

			// handle collisions here
			if (!collision_with_ground)
			{
				g1->flag = true;
				g2->flag = true;
			}
		}
	
		for (int i = 0; i < numc; i++)
		{
			dJointID c = dJointCreateContact(info->world, info->contacts, contact+i);
			dJointAttach(c, b1, b2);
		}
	}
}

void Play::tickWorld()
{
	Info info;
	info.world = world;
	info.contacts = collisions;
	info.wall = &wall;
	info.otherObjects = &otherObjects;

	dSpaceCollide(space, &info, near_callback);
	dWorldQuickStep(world, Tick);
	dJointGroupEmpty(collisions);

	for (unsigned int b=0; b < wall.size(); b++)
	{
		wall[b]->copy_body_position();
	}

	for (unsigned int b=0; b < otherObjects.size(); b++)
	{
		if (!otherObjects[b]->body) continue;
		otherObjects[b]->copy_body_position();
	}

	for (int i = 0; i < 5; i++)
	{
		city_tiles[i][0]->force_body();
		city_tiles[i][1]->force_body();
	}
}

