#include "Play.hpp"

#include <Graphics/Graphics.hpp>
#include <Vector/Quat.hpp>
#include <stdio.h>
#include <string>
#include <math.h>
#include <iostream>
#include <sstream>
#include <fstream>

using std::string;
using std::ostringstream;

#define MATH_PI 3.14159f
#define ABS(x) ((x) < 0 ? (-x) : x)

void whiteMaterial();
void redMaterial();
void greenMaterial();
void blackMaterial();
void cyanMaterial();
void yellowMaterial();
void greyMaterial();

Play::Play() {
	world = NULL;
	space = NULL;
	ground = NULL;
	collisions = NULL;
	rightIsPressed = false;
	leftIsPressed = false;
	upIsPressed = false;
	downIsPressed = false;
	zoomOutPressed = false;
	zoomInPressed = false;
	leftMouseButton = false;
	attackStarted = false;
	finalAttack = false;
	scoreScreenShowing = false;
	currentBlockRotation = 0;

	viewDistance = 5.0f;
	viewAngle = 0.0f;
	viewElevation = MATH_PI / 12;

	init();	
	startWorld();	
	initGame();
	simpleSpace = dHashSpaceCreate(NULL);
	simpleWorld = dWorldCreate();
	fontRef = Graphics::get_font("gentium.txf");
	assert(fontRef.ref);

	// init zone
	zone_length = 12;
	zone_width = 5;
	currentBlockSize = 0.5f;
	currentSelection = new Block(make_vector(currentBlockSize, 0.5f, 0.5f), make_vector(zone_width/2.0f, 0.51f, 0.0f), rotation(0.0f, make_vector(0.0f, 1.0f, 0.0f)));
	rotateCurrentBlock();
	
	intersectingWall = false;

	fp = fopen("objects.txt", "r");

	levelNumber = 0;
	money = 2000;
	loadObjects();

}

Play::~Play() {
}

Geometry::Geometry() {
}

Geometry::~Geometry() {
}

Sphere::Sphere(float rad, Vector3f pos, Quatf rot) : Geometry() {
	radius = rad;
	position = pos;
	rotation = rot;	
	body = NULL;
	geom = NULL;
	flag = false;
}

void Sphere::draw(){

	glPushMatrix();
	glTranslatef(position.x, position.y, position.z);
	glRotate(rotation);
	glScalef(radius, radius, radius);
	glColor3f(0.8, 0.0, 0.3);
	GLUquadric * myQuad = gluNewQuadric();
	gluSphere(myQuad, 1.0, 32, 32);
	glPopMatrix();


}

Block::Block(Vector3f rad, Vector3f pos, Quatf rot) : Geometry() {
	radius = rad;
	position = pos;
	rotation = rot;
	body = NULL;
	geom = NULL;
	flag = false;

	// init textures
	wallTexture = Graphics::get_texture("wall.png");
	if(wallTexture.ref){
		glColor3f(1, 1, 1);
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, wallTexture->obj);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	}
}

void Block::draw()
{
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, wallTexture->obj);

	glPushMatrix();
	glTranslatef(position.x, position.y, position.z);
	glRotate(rotation);
	
	Vector3f verts[8];
	for (unsigned int i = 0; i < 8; ++i) {
		verts[i] = -1.0f * radius;
		if (i & 1) verts[i].x *= -1.0f;
		if (i & 2) verts[i].y *= -1.0f;
		if (i & 4) verts[i].z *= -1.0f;
	}
	Vector4ui faces[6] = {
		make_vector( 0U, 4U, 6U, 2U ),
		make_vector( 0U, 2U, 3U, 1U ),
		make_vector( 0U, 1U, 5U, 4U ),
		make_vector( 7U, 6U, 4U, 5U ),
		make_vector( 7U, 3U, 2U, 6U ),
		make_vector( 7U, 5U, 1U, 3U ),
	};

	for (unsigned int i = 0; i < 6; i++)
	{
		Vector3f normal = cross_product(verts[faces[i].y] - verts[faces[i].x], verts[faces[i].z] - verts[faces[i].y]);	
		glBegin(GL_QUADS);
		glNormal3f(normal.x, normal.y, normal.z);
		glTexCoord2f(1,1);
		glVertex3f(verts[faces[i].x].x, verts[faces[i].x].y,verts[faces[i].x].z);
		
		glNormal3f(normal.x,  normal.y, normal.z);
		glTexCoord2f(1,0);
		glVertex3f(verts[faces[i].y].x, verts[faces[i].y].y,verts[faces[i].y].z);
		
		glNormal3f(normal.x,  normal.y, normal.z);
		glTexCoord2f(0,0);
		glVertex3f(verts[faces[i].z].x, verts[faces[i].z].y,verts[faces[i].z].z);
		
		glNormal3f(normal.x,  normal.y, normal.z);
		glTexCoord2f(0,1);
		glVertex3f(verts[faces[i].w].x, verts[faces[i].w].y,verts[faces[i].w].z);
		glEnd();
	}

	glPopMatrix();	
	glDisable(GL_TEXTURE_2D);		
}

void Play::init() {
	
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(75, Graphics::aspectf(), 0.01, 100);
	
	glEnable(GL_NORMALIZE);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	initLighting();

}

void Play::initLighting(){
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	
	float lightAmbient[] = {0.5f, 0.5f, 0.5f, 1};
	float lightDiffuse[] = {1.0f, 1.0f, 1.0f, 1.0f};
	float lightSpecular[] = {0.2f, 0.2f, 0.2f, 1};
	float lightPosition[] = {0, 5, 0, 1};
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lightAmbient);
	glLightfv(GL_LIGHT0, GL_POSITION, lightPosition);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR, lightSpecular);
	glLightf(GL_LIGHT0, GL_CONSTANT_ATTENUATION, 0.5f);
	glLightf(GL_LIGHT0, GL_LINEAR_ATTENUATION, 0.1f);

}

void Play::initGame()
{
	// create city	
	for (int i = 0; i < 5; i++)
	{
		for (int j = 0; j < 2; j++)
		{
			city_tiles[i][j] = new Block(make_vector(0.99f, 0.005f, 0.99f), make_vector(-3.0f + j * 2.0f, 0.005f, -4.0f + i * 2.0f), rotation(0.0f, make_vector(0.0f, 1.0f, 0.0f)));
			city_tiles[i][j]->density = 1.0f;
			city_tiles[i][j]->create_body(world, space);
		}
	}
}

bool Play::loadObjects(){

	// First line of each level: num objects, totalTime, money for winning, money per city, spy cost
	//usage:: objectType radius.x radius.y radius.z pos.x pos.y pos.z density initVel.x initVel.y initVel.z time
	//usage:: objectType == 0 represents sphere
	//usage:: objectType == 1 represents block
	int a;

	int objectType;
	Vector3f rad;
	Vector3f pos;
	float den;
	Vector3f initVel;
	float t;


	// a: num objects in next lvl
	
	{
		fscanf(fp, "%d", &a);
		if (a == 0) return false;
		fscanf(fp, "%d %d %d %d", &currLevel.totalTime, &currLevel.completionMoney, &currLevel.rewardMoney, &currLevel.spyCost);
		//clear the projectiles
		projectiles.clear();
		for (int i = 0; i < a; i++){
			fscanf(fp, "%d %f %f %f %f %f %f %f %f %f %f %f", &objectType, &(rad.x), &(rad.y), &(rad.z), &(pos.x), &(pos.y), &(pos.z),
				&den, &(initVel.x), &(initVel.y), &(initVel.z), &t);


			AttackObject * newAttack = new AttackObject();
			newAttack->initVelocity = make_vector(initVel.x, initVel.y, initVel.z);
			newAttack->time = t;

			if(objectType == 0){
				Sphere * newSphere = new Sphere(rad.x, make_vector(pos.x, pos.y, pos.z), rotation(0.0f, make_vector(0.0f, 1.0f, 0.0f)));
				newSphere->density = den;
				newAttack->object = newSphere;
			}
			else{
				Block * newBlock = new Block(make_vector(rad.x, rad.y, rad.z), make_vector(pos.x, pos.y, pos.z), rotation(0.0f,
					make_vector(0.0f, 1.0f, 0.0f)));
				newBlock->density = den;
				newAttack->object = newBlock;
			}
			projectiles.push_back(newAttack);
		}

	}

	// prepare next level
	timeToNextAttack = currLevel.totalTime;
	levelNumber++;
	spyShowing = false;
	
	return true;
}

void Play::adjustView()
{
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	Vector3f cameraPos = make_vector((float) (cos(viewElevation) * cos(viewAngle)), (float)sin(viewElevation), (float)(cos(viewElevation) * sin(viewAngle))) * viewDistance;
	gluLookAt(cameraPos.x, cameraPos.y, cameraPos.z, 0, 0, 0, 0, 1, 0);
}

void Play::drawGround()
{
	whiteMaterial();
	glBegin(GL_QUADS);
	glNormal3f(0, 1, 0);
	glVertex3f(-50, 0, -50);
	glNormal3f(0, 1, 0);
	glVertex3f(50, 0, -50);
	glNormal3f(0, 1, 0);
	glVertex3f(50, 0, 50);
	glNormal3f(0, 1, 0);
	glVertex3f(-50, 0, 50);
	glEnd();
	// draw city	
	for (int i = 0; i < 5; i++)
	{
		for (int j = 0; j < 2; j++)
		{
			if (city_tiles[i][j]->flag) 
			{
				redMaterial();
			}
			else
			{
				greenMaterial();
			}
			glBegin(GL_QUADS);
			float x = -4 + 2 * (j);
			float z = -5 + 2 * (i);
			glNormal3f(0, 1, 0);
			glVertex3f(x, 0.01f, z);
			glNormal3f(0, 1, 0);
			glVertex3f(x+2, 0.01f, z);
			glNormal3f(0, 1, 0);
			glVertex3f(x+2, 0.01f, z+2);
			glNormal3f(0, 1, 0);
			glVertex3f(x, 0.01f, z+2);
			glEnd();
			blackMaterial();
			glLineWidth(3.0f);
			glBegin(GL_LINE_STRIP);
			glVertex3f(x, 0.02f, z);
			glVertex3f(x+2, 0.02f, z);
			glVertex3f(x+2, 0.02f, z+2);
			glVertex3f(x, 0.02f, z+2);
			glVertex3f(x, 0.02f, z);
			glEnd();
		}
	}

	// draw building zone
	glLineWidth(3.0f);
	cyanMaterial();
	glBegin(GL_LINE_STRIP);
	glVertex3f(0, 0.01, -zone_length/2);
	glVertex3f(0, 0.01, zone_length/2);
	glVertex3f(zone_width, 0.01, zone_length/2);
	glVertex3f(zone_width, 0.01, -zone_length/2);
	glVertex3f(0, 0.01, -zone_length/2);
	glEnd();

}

void Play::drawObjects()
{
	unsigned int i=0;
	for(i=0; i<wall.size();i++){
		whiteMaterial();
		wall[i]->draw();
	}
	for(i=0; i<otherObjects.size();i++){
		whiteMaterial();
		otherObjects[i]->draw();
	}
}

void Play::drawWireframeBox()
{
	glPushMatrix();
	
	glTranslatef(currentSelection->position.x, currentSelection->position.y, currentSelection->position.z);
	glRotate(currentSelection->rotation);
	
	Vector3f verts[8];
	for (unsigned int i = 0; i < 8; ++i) {
		verts[i] = -1.0f * currentSelection->radius;
		if (i & 1) verts[i].x *= -1.0f;
		if (i & 2) verts[i].y *= -1.0f;
		if (i & 4) verts[i].z *= -1.0f;
	}
	Vector2ui lines[12] = {
		make_vector( 0U, 1U ),
		make_vector( 0U, 2U ),
		make_vector( 1U, 3U ),
		make_vector( 2U, 3U ),
		make_vector( 0U, 4U ),
		make_vector( 1U, 5U ),
		make_vector( 2U, 6U ),
		make_vector( 3U, 7U ),
		make_vector( 4U, 5U ),
		make_vector( 6U, 7U ),
		make_vector( 4U, 6U ),
		make_vector( 5U, 7U )
	};
	// decide color

	int cost = (int)round(currentBlockSize * BASE_COST);
	if (cost > money)
	{
		greyMaterial();
	}
	else if (intersectingWall)
	{
		redMaterial();
	}
	else 
	{
		greenMaterial();
	}

	for (unsigned int i = 0; i < 12; i++)
	{
		glBegin(GL_LINES);
		glVertex3f(verts[lines[i].x].x, verts[lines[i].x].y, verts[lines[i].x].z);
		glVertex3f(verts[lines[i].y].x, verts[lines[i].y].y, verts[lines[i].y].z);
		glEnd();
	}

	glPopMatrix();

}

void Play::drawText() 
{
	// switch to 2d projection
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_LIGHTING);
	glPushMatrix();
	glLoadIdentity();
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	glScaled(1.0/Graphics::aspectf(), 1.0, 1.0);
	glMatrixMode(GL_MODELVIEW);

	// draw money on top left
	ostringstream moneyStr;
	glColor3f(1.0f, 1.0f, 1.0f);
	moneyStr <<  "Money: $" << money;
	fontRef.ref->draw(moneyStr.str(), make_vector(-(float)Graphics::aspectf() + 0.05f, 0.9f), 0.1f);

	ostringstream costStr;
	costStr << "Cost: $" << (int)round(currentBlockSize * BASE_COST);
	fontRef.ref->draw(costStr.str(), make_vector(-(float)Graphics::aspectf() * 0.4f, 0.9f), 0.1f);
	// draw remaining time	
	ostringstream timeStr;
	int time = (int)round(timeToNextAttack);
	int secs = time % 60;
	
	timeStr << "Time to next attack: " << (time/60) << ":";
	if (secs < 10) 
	{
		timeStr << "0" << secs;
	}
	else
	{
		timeStr << secs;
	}

	glColor3f(1.0f, 1.0f, 0.0f);
	if (time < 10)
	{
		glColor3f(1.0f, 0.0f, 0.0f);
	}
	fontRef.ref->draw(timeStr.str(), make_vector(0.0f, 0.9f), 0.1f);
	// draw current level
	ostringstream levelStr;
	glColor3f(1.0f, 1.0f, 1.0f);
	levelStr << "Level: " << levelNumber;
	fontRef.ref->draw(levelStr.str(), make_vector(Graphics::aspectf() * 0.75f, 0.9f), 0.1f);

	// draw some instructions
	glColor3f(0.0f, 0.0f, 0.0f);
	fontRef.ref->draw("SPACE - place block   WSAD/FV - move block   QE - rotate   1/2/3 - adjust size", make_vector(Graphics::aspectf() * -0.95f, -0.85f), 0.08f);
	ostringstream infoStr;
	infoStr << "B - buy time ($100)    Z - spy on enemy ($" << currLevel.spyCost << ")     ENTER - start attack now";
	fontRef.ref->draw(infoStr.str(), make_vector(Graphics::aspectf() * -0.95f, -0.93f), 0.08f);

	if (scoreScreenShowing)
	{
		// draw the score screen
		glColor4f(0.0f, 0.5f, 0.7f, 0.7f);
		glBegin(GL_QUADS);
		glVertex2f(-Graphics::aspectf() * 0.8f, 0.7f);
		glVertex2f(Graphics::aspectf() * 0.8f, 0.7f);
		glVertex2f(Graphics::aspectf() * 0.8f, -0.4f);
		glVertex2f(-Graphics::aspectf() * 0.8f, -0.4f);
		glEnd();
		
		if (citiesAlive < 5)
		{
			glColor3f(1.0f, 0.0f, 0.0f);
			fontRef.ref->draw_center("You have failed.", make_vector(0.0f, 0.3f), 0.2f);
			fontRef.ref->draw_center("Game over!", make_vector(0.0f, 0.0f), 0.2f);
		}
		else
		{
			glColor3f(1.0f, 1.0f, 1.0f);
			fontRef.ref->draw_center("You survived. Prepare for next round!", make_vector(0.0f, 0.5f), 0.12f);
			ostringstream line1, line2, line3;
			line1 << "Completion Bonus : $" << currLevel.completionMoney;
			line2 << "Cities Survived       : $" << currLevel.rewardMoney << "*" << citiesAlive;
			line3 << "Total                         : $" << (currLevel.completionMoney + money + currLevel.rewardMoney * citiesAlive);
			
			glColor3f(1.0f, 0.5f, 0.0f);
			fontRef.ref->draw(line1.str(), make_vector(Graphics::aspectf() * -0.5f, 0.2f), 0.12f);
			fontRef.ref->draw(line2.str(), make_vector(Graphics::aspectf() * -0.5f, 0.05f), 0.12f);
			glColor3f(0.0f, 1.0f, 0.5f);
			fontRef.ref->draw(line3.str(), make_vector(Graphics::aspectf() * -0.5f, -0.10f), 0.12f);
			glColor3f(1.0f, 1.0f, 1.0f);
			fontRef.ref->draw_center("Press ENTER to continue", make_vector(0.0f, -0.25f), 0.12f);

		}

	}

	glPopMatrix();
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);
}

#define INTERVAL 0.01f

void Play::drawTrajectory()
{
	redMaterial();
	glLineWidth(3.0f);
	
	for (unsigned int i = 0; i < projectiles.size(); i++)
	{
		float x = projectiles[i]->object->position.x;
		float y = projectiles[i]->object->position.y;
		float z = projectiles[i]->object->position.z;
		float dx = projectiles[i]->initVelocity.x;
		float dy = projectiles[i]->initVelocity.y;
		float dz = projectiles[i]->initVelocity.z;
		glBegin(GL_LINE_STRIP);
		for (float t = 0.0f; t < 5.0f; t+=INTERVAL)
		{
			float px = x + dx * t;
			float py = y + dy * t + -5.0f * t * t;
			float pz = z + dz * t;
			glVertex3f(px, py, pz);
		}
		glEnd();
	}

}

void Play::draw() {
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	adjustView();
	// draw ground
	drawGround();
	drawObjects();
	if(!attackStarted){
		drawWireframeBox();
	}
	drawText();
	if (spyShowing)
	{
		drawTrajectory();
	}
}

void Play::materialize()
{
	// check if the user has money
	int cost = (int)round(currentBlockSize * BASE_COST);
	if (money < cost)
	{
		return;
	}
	money -= cost;
	currentSelection->density = 1.0f;
	currentSelection->create_body(world, space);


	//loop through the wall and get the max height
	/*
	unsigned int i=0;
	float maxHeight = currentSelection->radius.y + 0.01f;
	for(i=0; i<wall.size(); i++){
		Block * curr = (Block *) wall[i];
		if(curr->position.y + curr->radius.y > maxHeight){		
			maxHeight = curr->position.y + curr->radius.y;
		}
	}
*/
	//add selection to the wall
	wall.push_back(currentSelection);
	float xpos = currentSelection->position.x;
	float ypos = currentSelection->position.y;
	float zpos = currentSelection->position.z;

	//create new block for the player
	currentSelection = new Block(make_vector(currentBlockSize, 0.5f, 0.5f), make_vector(xpos, ypos, zpos), rotation(0.0f, make_vector(0.0f, 1.0f, 0.0f)));
	
	rotateCurrentBlock();	
}

void Play::rotateCurrentBlock()
{
	if (currentBlockRotation == 0)
	{
		currentSelection->radius.x = currentBlockSize;
	}
	else
	{
		currentSelection->radius.x = 0.5f;
	}

	if (currentBlockRotation == 1)
	{
		currentSelection->radius.z = currentBlockSize;
	}
	else
	{
		currentSelection->radius.z = 0.5f;
	}

	if (currentBlockRotation == 2)
	{
		currentSelection->radius.y = currentBlockSize;
	}
	else
	{
		currentSelection->radius.y = 0.5f;
	}

	if (currentSelection->position.y < currentSelection->radius.y + 0.01f)
	{
		currentSelection->position.y = currentSelection->radius.y + 0.01f;
	}
}

void Play::updateView(float elapsed_time)
{
	if (zoomOutPressed)
	{
		viewDistance += elapsed_time * 4;
		if (viewDistance > 20) viewDistance = 20;
	}
	else if (zoomInPressed)
	{
		viewDistance -= elapsed_time * 4;
		if (viewDistance < 2) viewDistance = 2;
	}


}

void intersectWallCallback(void *data, dGeomID o1, dGeomID o2)
{
	bool *iw = (bool *)data;	
	*iw = true;	
}

void Play::updateSelection(float elapsed_time)
{
	const float movement_speed = 1.5f;
	if (forwardIsPressed)
	{
		currentSelection->position.x -= elapsed_time * movement_speed;
		if (currentSelection->position.x - currentSelection->radius.x <= 0.01f)
		{
			currentSelection->position.x = currentSelection->radius.x + 0.01f;
		}
	}
	if (backIsPressed)
	{
		currentSelection->position.x += elapsed_time * movement_speed;
		if (currentSelection->position.x + currentSelection->radius.x >= zone_width - 0.01f)
		{
			currentSelection->position.x = zone_width - currentSelection->radius.x - 0.01f;
		}
	}
	
	if (leftIsPressed)
	{
		currentSelection->position.z += elapsed_time * movement_speed;
		if (currentSelection->position.z + currentSelection->radius.z >= zone_length / 2 - 0.01f)
		{
			currentSelection->position.z = zone_length/2 - currentSelection->radius.z - 0.01f;
		}
	}

	if (rightIsPressed)
	{
		currentSelection->position.z -= elapsed_time * movement_speed;
		if (currentSelection->position.z - currentSelection->radius.z <= -zone_length / 2 + 0.01f)
		{
			currentSelection->position.z = -zone_length/2 + currentSelection->radius.z + 0.01f;
		}
	}

	if (upIsPressed)
	{
		currentSelection->position.y += elapsed_time * movement_speed;
		if (currentSelection->position.y + currentSelection->radius.y >= 19.99f)
		{
			currentSelection->position.y = 19.99f - currentSelection->radius.y;
		}
	}

	if (downIsPressed)
	{
		currentSelection->position.y -= elapsed_time * movement_speed;
		if (currentSelection->position.y - currentSelection->radius.y < 0.01f)
		{
			currentSelection->position.y = currentSelection->radius.y + 0.01f;
		}
	}

	// check for intersection
	intersectingWall = false;
	// create geometry 
	dBodyID selBody = dBodyCreate(simpleWorld);
	dMass mass;
	dMassSetBox(&mass, 1.0f, currentSelection->radius.x * 2, currentSelection->radius.y * 2, currentSelection->radius.z * 2);
	dBodySetMass(selBody, &mass);
	dGeomID selGeom = dCreateBox(simpleSpace, currentSelection->radius.x * 2, currentSelection->radius.y * 2, currentSelection->radius.z * 2);
	dGeomSetBody(selGeom, selBody);
	dBodySetPosition(selBody, currentSelection->position.x, currentSelection->position.y, currentSelection->position.z);
	dQuaternion quat;
	quat[1] = currentSelection->rotation[0];
	quat[2] = currentSelection->rotation[1];
	quat[3] = currentSelection->rotation[2];
	quat[0] = currentSelection->rotation[3];
	dBodySetQuaternion(selBody, quat);	

	for (unsigned int i = 0; i < wall.size(); i++)
	{
		dContactGeom contact_array[5];
		int num_contact = dCollide(wall[i]->geom, selGeom, 5, contact_array, sizeof(dContactGeom));
		if (num_contact > 0)
		{
			intersectingWall = true;
			break;
		}
	}
	//dSpaceCollide2((dGeomID)space, selGeom, &intersectingWall, intersectWallCallback);
	
	dGeomDestroy(selGeom);
	dBodyDestroy(selBody);
}

void Play::updateAttack(float elapsed_time) {

	if (scoreScreenShowing) return;
	if (!attackStarted)
	{
		timeToNextAttack -= elapsed_time;
		if (timeToNextAttack <= 0)
		{
			timeToNextAttack = 0;
			startAttack();
		}
	}

	if(attackStarted){
		attackTimer += elapsed_time;
		unsigned int i=0;
		for(i =0; i<projectiles.size(); i++){
			AttackObject* curr = (AttackObject *)projectiles[i];
			if(attackTimer >= curr->time){
				otherObjects.push_back(curr->object);
				curr->object->create_body(world, space);
				dBodySetLinearVel(curr->object->body, curr->initVelocity.x, curr->initVelocity.y, curr->initVelocity.z);
				delete(curr);
				projectiles.erase(projectiles.begin() + i);
				i--;
				if (projectiles.size() == 0)
				{
					// this is the last attack
					finalAttackTimer = 0;
					finalAttack = true;
				}
			}
		}
	}	

	if (finalAttack)
	{
		finalAttackTimer += elapsed_time;
	}
}

bool outOfBounds(Vector3f pos, Vector3f rad)
{
	// check out of bounds
	if (pos.x + rad.x < -5) return true;
	if (pos.x < 0 && pos.z + rad.z < -6) return true;
	if (pos.x < 0 && pos.z - rad.z > 6) return true;
	return false;
}

void Play::checkObjects()
{
	if (finalAttack && finalAttackTimer > 5.0f)
	{
		bool settled = true;	
		for (unsigned int i = 0; i < wall.size(); i++)
		{
			Block *b = wall[i];
			const dReal *vel = dBodyGetLinearVel(b->body);
			if (ABS(vel[0]) > 0.05f || ABS(vel[1]) > 0.05f || ABS(vel[2]) > 0.05f)
			{
				settled = false;	
			}
		}
		
		if (settled)
		{
			// attack is over
			attackStarted = false;
			attackTimer = 0;
			finalAttack = false;
			finalAttackTimer = 0;
			// remove all attack objects	
			for (unsigned int i = 0; i < otherObjects.size(); i++)
			{
				Geometry *g = otherObjects[i];
				g->destroy_body();
				delete(g);
			}
			otherObjects.clear();
			// remove all objects that touched the city
			dGeomID selGeom = dCreateBox(simpleSpace, 10, 20, 20);
			dBodyID selBody = dBodyCreate(simpleWorld);
			dMass mass;
			dMassSetBox(&mass, 1.0f, 10, 20, 20);
			dBodySetMass(selBody, &mass);
			dGeomSetBody(selGeom, selBody);
			dBodySetPosition(selBody, -5, 10, 0);
			dBodySetData(selBody, (void*)42);	
			dQuaternion quat;
			quat[1] = 0.0f;
			quat[2] = 1.0f;
			quat[3] = 0.0f;
			quat[0] = 0.0f;
			dBodySetQuaternion(selBody, quat);

			for (unsigned int i = 0; i < wall.size(); i++)
			{
				dContactGeom contacts[5];
				Block *b = wall[i];
				int num_contact = dCollide(selGeom, b->geom, 5, contacts, sizeof(dContactGeom));
				if (num_contact > 0)
				{
					// remove block from wall
					b->destroy_body();
					delete(b);
					wall.erase(wall.begin() + i);
					i--;
				}
			}
			dGeomDestroy(selGeom);
			dBodyDestroy(selBody);

			roundOver();
		}
	}
}

void Play::roundOver()
{
	// the round is over
	// display the user's score, and advance to the next level by calling 
	// loadobjects. if loadobjects is false, the game is over
	// if more than 5 cities have been destroyed, the game is also over
	scoreScreenShowing = true;
	citiesAlive = 0;
	for (int i = 0; i < 5; i++)
	{	
		for (int j = 0; j < 2; j++)
		{
			if (!city_tiles[i][j]->flag)
			{
				citiesAlive++;
			}
		}
	}
}

void Play::update(float elapsed_time) {
	//Update game state here.
	tickWorld();
	updateView(elapsed_time);
	updateSelection(elapsed_time);
	updateAttack(elapsed_time);
	checkObjects();
}

void Play::startAttack()
{
	attackStarted = true;
	attackTimer = 0;
	finalAttack = false;
	finalAttackTimer = 0;
	timeToNextAttack = 0;
}

void Play::beginNextLevel()
{ 
	// start next level
	// add money, refresh dead cities
	money += currLevel.completionMoney + currLevel.rewardMoney * citiesAlive;

	for (int i = 0; i < 5; i++)
	{
		city_tiles[i][0]->flag = false;
		city_tiles[i][1]->flag = false;
	}

	if (!loadObjects())
	{
		printf("You have completed the last level, game over.\n");
		quit_screen(this);
	};

}

void Play::handle_event(SDL_Event const &event) {

	//Handle events here:
	if (event.type == SDL_KEYDOWN) {
		if (event.key.keysym.sym == SDLK_ESCAPE) {
			//escape quits.
			quit_screen(this);
		}

		if (scoreScreenShowing) {
			if (event.key.keysym.sym == SDLK_RETURN)
			{
				if (citiesAlive < 5)
				{
					quit_screen(this);
				}
				scoreScreenShowing = false;
				beginNextLevel();
			}
			return;
		}

		if (event.key.keysym.sym == SDLK_SPACE){
			if (!intersectingWall && !attackStarted) 
				materialize();
		}

		if (event.key.keysym.sym == SDLK_RETURN){
			startAttack();
		}

		if (event.key.keysym.sym == SDLK_b)
		{
			if (!attackStarted && !finalAttack && !scoreScreenShowing && money >= 100)
			{
				money -= 100;
				timeToNextAttack += 15;
			}
		}

		if (event.key.keysym.sym == SDLK_z)
		{
			if (!spyShowing && !attackStarted && !finalAttack && !scoreScreenShowing && money >= currLevel.spyCost)
			{
				money -= currLevel.spyCost;
				spyShowing = true;
			}
		}

		if (event.key.keysym.sym == SDLK_1)
		{
			currentBlockSize = 0.25f;
			rotateCurrentBlock();
		}
		if (event.key.keysym.sym == SDLK_2)
		{
			currentBlockSize = 0.5f;
			rotateCurrentBlock();
		}
		if (event.key.keysym.sym == SDLK_3)
		{
			currentBlockSize = 1.0f;
			rotateCurrentBlock();
		}
		if (event.key.keysym.sym == SDLK_q)
		{
			currentBlockRotation--;
			if (currentBlockRotation < 0) currentBlockRotation = 2;
			rotateCurrentBlock();	
		}
		if (event.key.keysym.sym == SDLK_e)
		{
			currentBlockRotation++;
			if (currentBlockRotation > 2) currentBlockRotation = 0;
			rotateCurrentBlock();	
		}
	}

	Uint8 *keystate = SDL_GetKeyState(NULL);
	if (keystate[SDLK_ESCAPE]) quit_screen(this);

	rightIsPressed = keystate[SDLK_d];
	leftIsPressed = keystate[SDLK_a];
	forwardIsPressed = keystate[SDLK_w];
	backIsPressed = keystate[SDLK_s];
	upIsPressed = keystate[SDLK_f];
	downIsPressed = keystate[SDLK_v];


	zoomOutPressed = keystate[SDLK_PAGEUP];
	zoomInPressed = keystate[SDLK_PAGEDOWN];

	// handle mouse events
	if (event.type == SDL_MOUSEBUTTONDOWN)
	{
		if (event.button.button == SDL_BUTTON_LEFT)
		{
			leftMouseButton = true;
		}
	}

	if (event.type == SDL_MOUSEBUTTONUP)
	{
		if (event.button.button == SDL_BUTTON_LEFT)
		{
			leftMouseButton = false;
		}
		else if (event.button.button == SDL_BUTTON_WHEELUP)
		{
			viewDistance -= 0.15f;
			if (viewDistance < 2) viewDistance = 2;
		}
		else if (event.button.button == SDL_BUTTON_WHEELDOWN)
		{
			viewDistance += 0.15f;
			if (viewDistance > 20) viewDistance = 20;
		}
	}

	if (event.type == SDL_MOUSEMOTION)
	{
		// get mouse offset
		if (leftMouseButton)
		{
			float xmov = (float)event.motion.xrel;
			float ymov = (float)event.motion.yrel;
			viewAngle -= xmov * 0.003f;
			viewElevation += ymov * 0.003f;
			if (viewElevation > MATH_PI/3) viewElevation = MATH_PI/3;
			if (viewElevation < MATH_PI/15) viewElevation = MATH_PI/15;
		}
	}

}

void whiteMaterial()
{
	float ambient[] = {1.0f, 1.0f, 1.0f};
	float specular[] = {0.2f, 0.2f, 0.2f, 1.0f};
	float diffuse[] = {1.0f, 1.0f, 1.0f, 1.0f};
	float shininess[] = {3.0f};
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, shininess);
}

void blackMaterial()
{
	float ambient[] = {0.1f, 0.1f, 0.1f};
	float specular[] = {0.2f, 0.2f, 0.2f, 1.0f};
	float diffuse[] = {0.0f, 0.0f, 0.0f, 1.0f};
	float shininess[] = {1.0f};
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, shininess);
}

void greyMaterial()
{
	float ambient[] = {0.25f, 0.25f, 0.25f};
	float specular[] = {0.2f, 0.2f, 0.2f, 1.0f};
	float diffuse[] = {0.5f, 0.5f, 0.5f, 1.0f};
	float shininess[] = {1.0f};
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, shininess);
}

void redMaterial()
{
	float ambient[] = {0.5f, 0.0f, 0.0f};
	float specular[] = {0.2f, 0.2f, 0.2f, 1.0f};
	float diffuse[] = {1.0f, 0.0f, 0.0f, 1.0f};
	float shininess[] = {1.0f};
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, shininess);
}

void greenMaterial()
{
	float ambient[] = {0.0f, 0.5f, 0.0f};
	float specular[] = {0.2f, 0.2f, 0.2f, 1.0f};
	float diffuse[] = {0.0f, 1.0f, 0.0f, 1.0f};
	float shininess[] = {1.0f};
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, shininess);
}

void cyanMaterial()
{
	float ambient[] = {0.0f, 0.5f, 0.5f};
	float specular[] = {0.2f, 0.2f, 0.2f, 1.0f};
	float diffuse[] = {0.0f, 1.0f, 1.0f, 1.0f};
	float shininess[] = {1.0f};
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, shininess);
}

void yellowMaterial()
{
	float ambient[] = {0.5f, 0.5f, 0.0f};
	float specular[] = {0.2f, 0.2f, 0.2f, 1.0f};
	float diffuse[] = {1.0f, 1.0f, 0.0f, 1.0f};
	float shininess[] = {1.0f};
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, shininess);
}
