#ifndef PLAY_HPP
#define PLAY_HPP

#include <Basis/Screen.hpp>

#include <Vector/Quat.hpp>
#include <Vector/Box.hpp>

#include <vector>
#include <Graphics/Graphics.hpp>
#include <Graphics/Texture.hpp>
#include <Graphics/Font.hpp>
#include <Vector/VectorGL.hpp>
#include <Vector/QuatGL.hpp>
#include <tr1/unordered_map>
#include <utility>

#include <ode/ode.h>

#define SHAPE_SPHERE 1
#define SHAPE_BLOCK 2
#define GROUND_CONSTANT 21893
#define BASE_COST 200

using std::vector;
using std::tr1::unordered_map;
using std::make_pair;
using std::pair;

const float Tick = 0.01f;

class Level {
public:
	int totalTime;
	int completionMoney;
	int rewardMoney;
	int spyCost;
};
	
class Geometry{
public:

	/*variables*/
	Vector3f position;
	Quatf rotation;
	float density;
	bool flag;

	dBodyID body;
	dGeomID geom;
	
	/*functions*/
	Geometry();
	virtual ~Geometry();

	virtual void draw() = 0;
	virtual void create_body(dWorldID world, dSpaceID space) = 0;
	void force_body();
	void copy_body_position();
	void destroy_body();
	virtual int getShape() = 0;
};

class Sphere : public Geometry{
public:
	/*variables*/
	float radius;
	
	/*functions*/
	Sphere(float rad, Vector3f pos, Quatf rot);
	~Sphere()
	{}
	virtual void draw();
	virtual void create_body(dWorldID world, dSpaceID space);
	virtual int getShape()
	{
		return SHAPE_SPHERE;
	}
	
};

class Block : public Geometry{
public:
	/*variables*/
	Vector3f radius;
	Graphics::TextureRef wallTexture;
	
	/*functions*/
	Block(Vector3f rad, Vector3f pos, Quatf rot);
	~Block()	
	{}
	virtual void draw();
	virtual void create_body(dWorldID world, dSpaceID space);
	virtual int getShape()
	{
		return SHAPE_BLOCK;
	}

};

class AttackObject {
public:
	Geometry *object;
	Vector3f initVelocity;
	float time;

};

class Play : public Screen {
public:
	Play();
	virtual ~Play();
	virtual void draw();
	virtual void update(float elapsed_time);
	virtual void handle_event(SDL_Event const &event);

	virtual bool loadObjects();
	void init();
	void initLighting();
	void initGame();
	
	void startWorld();
	void tickWorld();
	void stopWorld();

	void materialize();
	void startAttack();
	void beginNextLevel();
	void adjustView();
	void drawGround();
	void drawObjects();
	void drawWireframeBox();
	void drawText();
	void drawTrajectory();
	void updateView(float elapsed_time);
	void updateSelection(float elapsed_time);
	void updateAttack(float elapsed_time);
	void checkObjects();
	void roundOver();
	void rotateCurrentBlock();
	float viewDistance;
	float viewAngle;
	float viewElevation;

	dWorldID world, simpleWorld;
	dSpaceID space, simpleSpace;
	dGeomID ground;
	dJointGroupID collisions;
	float sim_acc;

	vector < Block* > wall;
	vector < Geometry* > otherObjects;
	vector < AttackObject* > projectiles;
	Block *currentSelection;
	float currentBlockSize;
	int currentBlockRotation;
	bool intersectingWall;

	// control vectors
	bool rightIsPressed;
	bool leftIsPressed;
	bool upIsPressed;
	bool downIsPressed;
	bool forwardIsPressed;
	bool backIsPressed;
	bool zoomOutPressed;
	bool zoomInPressed;
	bool leftMouseButton;

	// city state
	Block * city_tiles[5][2];

	// game state
	float zone_width;
	float zone_length;
	bool attackStarted;
	bool finalAttack;
	bool scoreScreenShowing;
	bool spyShowing;
	float attackTimer;
	float finalAttackTimer;
	int money;
	int levelNumber;
	float timeToNextAttack;
	int citiesAlive;	
	Level currLevel;

	// other state
	Graphics::FontRef fontRef;
	FILE * fp;
};

#endif //PLAY_HPP
